/* Copyright 2006-2007 (C) Raster Software Vigo (Sergio Costas)
*
* This file is part of GtkPSproc
*
* GtkPSproc is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* GtkPSproc is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#define _ gettext
#include "variables.h"

#include <cups/cups.h>

struct printer_conf *current_config=NULL;
gchar pulsado;

void set_configuration(gboolean t1,gboolean t2,gboolean t3, gboolean bmanual,
					 gboolean binvert, gboolean binvertodd, gboolean binverteven,
					 gboolean bactivate_buttons) {

	GtkWidget *type1,*type2,*type3,*manual,*invert,*invertodd,*inverteven,*ok,*cancel;

#ifdef DEBUG
	g_print("Set_configuration\n");
#endif

	type1=glade_xml_get_widget(xml,"canontype");
	type2=glade_xml_get_widget(xml,"hptype");
	type3=glade_xml_get_widget(xml,"lasertype");
	manual=glade_xml_get_widget(xml,"manual");
	invert=glade_xml_get_widget(xml,"invert");
	invertodd=glade_xml_get_widget(xml,"invertodd");
	inverteven=glade_xml_get_widget(xml,"inverteven");
	ok=glade_xml_get_widget(xml,"confok");
	cancel=glade_xml_get_widget(xml,"confcancel");

	if(bactivate_buttons)
		gtk_widget_set_sensitive(ok,1);
	else
		gtk_widget_set_sensitive(ok,0);

	if((bactivate_buttons)&&(!first_time))
		gtk_widget_set_sensitive(cancel,1);
	else
		gtk_widget_set_sensitive(cancel,0);

	gtk_widget_set_sensitive(invert,1);
	gtk_widget_set_sensitive(invertodd,1);
	gtk_widget_set_sensitive(inverteven,1);

	if(binvert)
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(invert),1);
	else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(invert),0);

	if(binvertodd)
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(invertodd),1);
	else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(invertodd),0);

	if(binverteven)
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(inverteven),1);
	else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(inverteven),0);

	if(bmanual) {
		gtk_widget_set_sensitive(invert,1);
		gtk_widget_set_sensitive(invertodd,1);
		gtk_widget_set_sensitive(inverteven,1);
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(manual),1);
	} else {
		gtk_widget_set_sensitive(invert,0);
		gtk_widget_set_sensitive(invertodd,0);
		gtk_widget_set_sensitive(inverteven,0);
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(manual),0);
	}

	if(t1) {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type1),1);
	} else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type1),0);

	if(t2) {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type2),1);
	} else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type2),0);

	if(t3) {
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type3),1);
	} else
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(type3),0);

	if (t1|t2|t3|bmanual) {
		pulsado=1;
	} else {
		pulsado=0;
	}
}

void store_current_conf() {
	
	GtkWidget *invert,*invertodd,*inverteven,*margenes;
	
#ifdef DEBUG
	g_print("store_current_conf\n");
#endif
	
	if (current_config!=NULL) {
#ifdef DEBUG
	g_print("current_config not NULL\n");
#endif
		invert=glade_xml_get_widget(xml,"invert");
		invertodd=glade_xml_get_widget(xml,"invertodd");
		inverteven=glade_xml_get_widget(xml,"inverteven");
		margenes=glade_xml_get_widget(xml,"margen");
	
		current_config->margen=(gchar) gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(margenes));
	
		current_config->configuracion=0;
	
		if (GTK_TOGGLE_BUTTON (invert)->active)
			current_config->configuracion+=1;
		if (GTK_TOGGLE_BUTTON (invertodd)->active)
			current_config->configuracion+=2;
		if (GTK_TOGGLE_BUTTON (inverteven)->active)
			current_config->configuracion+=4;
		current_config->configurada=pulsado;
	}
#ifdef DEBUG
	else
		g_print("current_config NULL\n");
#endif
}

void on_confprinter_changed(GtkToggleButton *togglebutton, gpointer user_data) {
	
#ifdef DEBUG
	g_print("on_confprinter_changed\n");
#endif
	
	store_current_conf();	
	set_values_config();
	
}

void set_values_config() {
	
	gboolean t1,t2,t3,t4,invert,invertodd,inverteven;
	GtkWidget *margenes;

	gchar *curprinter_char;
	struct printer_conf *curprinter;
	
#ifdef DEBUG
	g_print("set_values_config\n");
#endif
	
	margenes=glade_xml_get_widget(xml,"margen");

	curprinter_char=gtk_combo_box_get_active_text(GTK_COMBO_BOX(wconfprinter));
	curprinter=set_printer((guchar *)curprinter_char,temp_config_list);
	current_config=curprinter;
	
	t1=FALSE;
	t2=FALSE;
	t3=FALSE;
	t4=FALSE;

	pulsado=0;
	
	if ((curprinter!=NULL)&&(curprinter->configurada)) {
		pulsado=1;
		if(curprinter->configuracion&0x01)
			invert=TRUE;
		else
			invert=FALSE;
	
		if(curprinter->configuracion&0x02)
			invertodd=TRUE;
		else
			invertodd=FALSE;
	
		if(curprinter->configuracion&0x04)
			inverteven=TRUE;
		else
			inverteven=FALSE;
	
		switch(curprinter->configuracion&0x07) {
		case 1:
			t1=TRUE;
		break;
		case 3:
			t2=TRUE;
		break;
		case 4:
			t3=TRUE;
		break;
		default:
			t4=TRUE;
		break;
		}
	}

	set_configuration(t1,t2,t3,t4,invert,invertodd,inverteven,TRUE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(margenes),(gfloat)margen);	
	
}


void on_configurar_clicked (GtkButton *button, gpointer user_data) {

	struct printer_conf *pointer,*tempo;
	cups_dest_t *printers;
	int num,bucle;
	GtkWidget *visor;
	
#ifdef DEBUG
	g_print("on_configurar_clicked\n");
#endif

	gtk_widget_show(config_window);
	
	visor=glade_xml_get_widget(xml,"combo1");
	
	temp_config_list=NULL;
	
	// we create a temporal copy of the configuration list to work with.
	
	for(pointer=configuration_list;pointer!=NULL;pointer=pointer->next) {
		tempo=(struct printer_conf *)g_malloc(sizeof(struct printer_conf));
		tempo->next=temp_config_list;
		temp_config_list=tempo;
		tempo->name=g_string_new(pointer->name->str);
		tempo->margen=pointer->margen;
		tempo->configuracion=pointer->configuracion;
		tempo->configurada=pointer->configurada;
		tempo->isdefault=pointer->isdefault;
	}
	
	// set the VISOR entry
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(visor)->entry),visor_ps->str);
	
	// fill the printer ComboBox
	
	gtk_combo_box_set_active(GTK_COMBO_BOX(wconfprinter),0);
	
	set_values_config();
	
}

void delete_conf_tree(struct printer_conf *lista) {
	
	struct printer_conf *pointer,*pointer2;

#ifdef DEBUG
	g_print("delete_conf_tree\n");
#endif

	for(pointer=lista;pointer!=NULL;pointer=pointer2) {
		pointer2=pointer->next;
		g_string_free(pointer->name,TRUE);
		g_free(pointer);
	}
}

void on_confok_clicked 	(GtkButton	*button, gpointer user_data) {

	GtkWidget *visor;
	GString *cadena;
	struct printer_conf *pointer;
	int position;

#ifdef DEBUG
	g_print("on_confok_clicked\n");
#endif
	
	store_current_conf();
	
	delete_conf_tree(configuration_list);
	configuration_list=temp_config_list;
	temp_config_list=NULL;

	visor=glade_xml_get_widget(xml,"combo-entry1");
	
	g_string_printf(visor_ps,"%s",gtk_entry_get_text(GTK_ENTRY(visor)));
	
	check_for_visor();
	
	save_config();
	
	set_preview_buttons();
	gtk_widget_hide(config_window);

}

void on_confcancel_clicked (GtkButton *button, gpointer user_data) {

#ifdef DEBUG
	g_print("on_confcancel_clicked\n");
#endif

	delete_conf_tree(temp_config_list);
	gtk_widget_hide(config_window);

}

void on_canontype_released (GtkButton *button, gpointer user_data) {
	
	set_configuration(TRUE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,TRUE);
}


void on_hptype_released (GtkButton *button, gpointer user_data) {
	
	set_configuration(FALSE,TRUE,FALSE,FALSE,TRUE,TRUE,FALSE,TRUE);
}


void on_lasertype_released (GtkButton *button, gpointer user_data) {
	
	set_configuration(FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,TRUE,TRUE);
}


void on_manual_released (GtkButton *button, gpointer user_data) {
	
	set_configuration(FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,TRUE);
}
