/* Copyright 2006-2007 (C) Raster Software Vigo (Sergio Costas)
*
* This file is part of GtkPSproc
*
* GtkPSproc is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* GtkPSproc is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#define _ gettext

#include "variables.h"

struct papertypes sizes[] = {
   { "a3", 842, 1191 },
   { "a4", 595, 842 },
   { "a5", 421, 595 },
   { "b5", 516, 729 },
   { "letter", 612, 792 },
   { "legal", 612, 1008 },
   { "ledger", 1224, 792 },
   { "tabloid", 792, 1224 },
   { "statement", 396, 612 },
   { "executive", 540, 720 },
   { "folio", 612, 936 },
   { "quarto", 610, 780 },
   { "10x14", 720, 1008 },
   { NULL, 0, 0 }
};

int readline(FILE *fichero,gchar *buffer) {
	
	gchar carac;
	gchar first;
	gchar *pointer;
	gint num;
	int value;
	
	pointer=buffer;
	
	num=0;
	first=1;
	while(1) {
		value=fgetc(fichero);
		if (value==EOF) {
			break;
		}
		carac=(gchar) value;
		if (first && ((carac==10)||(carac==13))) { // jump over the LF from a previous CR LF
			continue; // and blank lines
		}
		first=0;
		if ((carac==10)||(carac==13)) {
			break;
		} // end of line
		
		if (num<255) {
			*pointer=carac; // store the character
			pointer++;
			num++;
		}
	}
	*pointer=0; // zero terminated string
	return (num);
}

gint checkatend(gchar *buffer) {
	
	gchar buf2[256];
	
	sscanf(buffer,"%s",buf2);
	if (0==strcmp(buf2,"(atend)"))
		return 1;
	else
		return 0;
	
}

int get_ps_size(GString *filename,struct pssize *ps_size) {
	
	FILE *fichero;
	gint atend;
	gchar buffer[256],b1[256],b2[256],b3[256],b4[256];
	int tamano;
	struct papertypes *pointer;
	
	ps_size->found=0;
	
	ps_size->width=595;
	ps_size->heigth=842; // by default is an A4
	
	fichero=fopen(filename->str,"r");
	if (fichero==NULL) {
		g_print("Error\n");
		return -1;
	}
	
	atend=0;
	
	do {
		tamano=readline(fichero,buffer);
		if (tamano==0) {
			break;
		}

		if (((!(ps_size->found&DOCUMENTMEDIA)))&&(0==strncmp(buffer,"%%DocumentMedia:",16))) {
			ps_size->found|=DOCUMENTMEDIA;
			sscanf(buffer+17,"%s %s %s",b1,b2,b3);
			ps_size->dwidth=atoi(b2);
			ps_size->dheigth=atoi(b3);
		}
		
		if (((!(ps_size->found&BOUNDINGBOX)))&&(0==strncmp(buffer,"%%BoundingBox:",14))) {
			if (checkatend(buffer+15)) {
				ps_size->found|=BOUNDIGATEND;
				continue;
			}
			ps_size->found|=BOUNDINGBOX;
			sscanf(buffer+15,"%s %s %s %s",b1,b2,b3,b4);
			ps_size->bwidth=atoi(b3)+atoi(b1);
			ps_size->bheigth=atoi(b4)+atoi(b2); // cross your fingers...
		}
		
		if (((!(ps_size->found&DOCPAPERSIZES)))&&(0==strncmp(buffer,"%%DocumentPaperSizes:",21))) {
			ps_size->found|=DOCPAPERSIZES;
			sscanf(buffer+22,"%s",b1);
			ps_size->pwidth=595;
			ps_size->pheigth=842; // by default is an A4
			for(pointer=sizes;pointer->name!=NULL;pointer++) {
				if (0==strcasecmp(pointer->name,b1)) {
					ps_size->pwidth=pointer->width;
					ps_size->pheigth=pointer->heigth;
					break;
				}
			}
		}
		
		if ((!(ps_size->found&PAGESIZE))&&((0==strncmp(buffer,"%%BeginFeature:",15)))) {
			ps_size->found|=PAGESIZE;
			sscanf(buffer+16,"%s %s",b1,b2);
			if (0==strcmp(b1,"*PageSize")) {
				ps_size->swidth=595;
				ps_size->sheigth=842; // by default is an A4
				for(pointer=sizes;pointer->name!=NULL;pointer++) {
					if (0==strcasecmp(pointer->name,b1)) {
						ps_size->swidth=pointer->width;
						ps_size->sheigth=pointer->heigth;
						break;
					}
				}
			}
			break;
		}			
		
		if ((0==strncmp(buffer,"%%EndComments",13))&&((ps_size->found&DOCUMENTMEDIA)||(ps_size->found&DOCPAPERSIZES))) {
			break;
		}
		
	} while(tamano!=0);
	
	fclose(fichero);
	
	// choose the better size
	
	ps_size->width=595;
	ps_size->heigth=842; // by default is an A4
	
	if ((ps_size->found)&DOCUMENTMEDIA) { // best is DocumentMedia
		ps_size->width=ps_size->dwidth;
		ps_size->heigth=ps_size->dheigth;
		return 0;
	}
	
	if ((ps_size->found)&DOCPAPERSIZES) { // second best is DocPaperSizes
		ps_size->width=ps_size->pwidth;
		ps_size->heigth=ps_size->pheigth;
		return 0;
	}
	
	if ((ps_size->found)&PAGESIZE) { // third best is PAGESIZE from BEGINFEATURE
		ps_size->width=ps_size->swidth; // made specifically for OpenOffice (grrrrr)
		ps_size->heigth=ps_size->sheigth;
		return 0;
	}
	
	if ((ps_size->found)&BOUNDINGBOX) { // if not, use the BoundingBox values
		ps_size->width=ps_size->bwidth; // and cross your fingers...
		ps_size->heigth=ps_size->bheigth;
		return 0;
	} // if not, A4
	
	return 0;
}
