/* Copyright 2006-2007 (C) Raster Software Vigo (Sergio Costas)
*
* This file is part of GtkPSproc
*
* GtkPSproc is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* GtkPSproc is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#define _ gettext

#include "variables.h"

#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <cups/cups.h>

#define DEBUG2

GtkWidget *mainwindow=NULL;
GtkWidget *evenpagesOK=NULL;
GtkWidget *about_window=NULL;
GtkWidget *first_time_window=NULL;
GtkWidget *config_window=NULL;
GtkWidget *preparing_window=NULL;
GtkWidget *noviewer_window=NULL;
GtkWidget *noviewer2_window=NULL;

GtkPrintUnixDialog *wprinterdialog=NULL;
GtkPrinter *impresora_actual=NULL;
GtkPageSetup *hoja_actual=NULL;
GtkPrintSettings *conf_actual=NULL;
GtkWidget *wvisor=NULL;
GtkWidget *wconfprinter=NULL;
gint num_copias;

gint copias_final_odd;
gint copias_final_even;
gboolean dialogo;

GString *ficherops; /* filename for PS file */
guchar configuracion,margen,first_time,noborrar;
GString *parametros,*visor_ps,*job_ps;

gint to_delete,viewer_found;

GladeXML *xml;

struct printer_conf *configuration_list,*temp_config_list;

struct pssize thepapersize;

gint read_param(gint n_parametro,char *argv[]) {
	
	gchar *parametro;
	
	parametro=argv[n_parametro];
	n_parametro++;
	
	if(parametro[0]!='-') {
		if(ficherops==NULL) /* only the first filename */
			ficherops=g_string_new(parametro); /* if it's not an option, it's the filename */
		return n_parametro;
	}
	
	switch(parametro[1]) {
		case 'E':
		case 'p':
		case 'l':
			g_string_append_printf(parametros,"%s ",parametro); /* theses options hasn't parameters */
		break;
		case 'C':
		case 'J':
		case 'T':
			if(parametro[2]==0) {
				g_string_printf(job_ps,argv[n_parametro]);
				n_parametro++;
			} else {
				g_string_printf(job_ps,parametro+2);
			}
		break;
		case 'P':
		case 'o':
			if(parametro[2]==0) { /* the parameter isn't glued to the option */
				g_string_append_printf(parametros,"%s %s ",parametro,argv[n_parametro]);
				n_parametro++;
			} else
				g_string_append(parametros,parametro); /* the parameter is glued to the option */
		break;
		case 'r':
		case 'c':
		case 'd':
		case 'f':
		case 'g':
		case 'm':
		case 'n':
		case 'v':
		case 't':
			printf(_("WARNING: option %s not supported\n"),parametro);
		break;
		case 'i':
		case 'w':
			printf(_("WARNING: option %s not supported\n"),parametro);
			if(parametro[2]!=0) /* the parameter isn't glued to the option */
				n_parametro++;
		break;
		case '#':
			if(parametro[2]!=0) { /* the parameter isn't glued to the option */
				num_copias=atoi(parametro+2);
				if((num_copias<1)||(num_copias>100))
					num_copias=1;
			} else {
				num_copias=atoi(argv[n_parametro]);
				if((num_copias<1)||(num_copias>100))
					num_copias=1;
				n_parametro++;
			}
		break;
#ifdef DEBUG2
		case 'a': /* special option for debug: don't delete the temporal files */
			noborrar=1;
		break;
#endif
		default:
			printf(_("ERROR: unknown parameter %s. Exiting\n"),parametro);
			exit(1);
		break;
	}
	
	return (n_parametro);
}

void show_error(char *mensaje) {

	FILE *fichero;

	fichero=fopen("/var/tmp/main.txt","a");
	fprintf(fichero,mensaje);
	fclose(fichero);

}

void procesa_parametros(int argc,char *argv[]) {

	gint parametro;
	char cadena[1024];
	
	ficherops=NULL;
	job_ps=g_string_new(_("Untitled job"));
	
	noborrar=0;
	
	parametros=g_string_new("");

	sprintf(cadena,"N parametros: %d\n",argc);
	show_error(cadena);

	if(argc<2) {
		show_error("stdin\n");
		ficherops=g_string_new("/dev/stdin");
		return;
	}
	
	parametro=1;

	while(parametro<argc) {
		sprintf(cadena,"parametro %d: %s\n",parametro,argv[parametro]);
		show_error(cadena);
		parametro=read_param(parametro,argv);
	}

	if(ficherops==NULL) { /* if the filename hasn't been set, we use stdin */
		ficherops=g_string_new("/dev/stdin");
		show_error("uso stdin\n");
	}
}

int check_visor(gchar *programa) {
	
	GString *cadena;
	int retorno;
	
	cadena=g_string_new("");
	retorno=1;
	g_string_printf(cadena,"/usr/bin/%s",programa);
	retorno*=checkexist(cadena->str);
	g_string_printf(cadena,"/usr/local/bin/%s",programa);
	retorno*=checkexist(cadena->str);
	g_string_printf(cadena,"/usr/bin/X11/%s",programa);
	retorno*=checkexist(cadena->str);
	g_string_printf(cadena,"/usr/X11R6/bin/%s",programa);
	retorno*=checkexist(cadena->str);
	if(retorno==0)
		return 0; // the file exists in, at least, one directory
	else
		return 1;
}

struct printer_conf *set_printer(guchar *printer_name,struct printer_conf *list) {
	
	struct printer_conf *pointer;
	
	configuracion=0;
	margen=8;
	
#ifdef DEBUG
		g_print("searching printer %s %d\n",printer_name,list);
#endif

	for(pointer=list;pointer!=NULL;pointer=pointer->next) {
#ifdef DEBUG
		g_print("Comparing %s with %s ",pointer->name->str,printer_name);
#endif
		if (!strcmp(pointer->name->str,(gchar *)printer_name)) {
#ifdef DEBUG
		g_print("OK\n");
#endif
			configuracion=pointer->configuracion;
			margen=pointer->margen;
			return pointer;
		}
#ifdef DEBUG
		g_print("NO\n");
#endif
	}
	return NULL;
	
}

void save_config() {

	GString *path;
	char *home;
	FILE *fichero;
	struct printer_conf *pointer;
	
	path=g_string_new(getenv("HOME"));
	g_string_append(path,"/.gtkpsproc");
	
	fichero=fopen(path->str,"w");
	if (fichero==NULL) {
		return;
	}

	fprintf(fichero,"viewer:%s\n",visor_ps->str);
	for(pointer=configuration_list;pointer!=NULL;pointer=pointer->next) {
		if (pointer->configurada) {
			fprintf(fichero,"printer:%s\n",pointer->name->str);
			fprintf(fichero,"margin:%d\n",pointer->margen);
			fprintf(fichero,"config:%d\n",pointer->configuracion);
		}
	}
	fclose(fichero);
}

void load_config() {

	char *buffer=NULL;
	FILE *fichero;
	int size,bsize=0;
	gchar impresora[33];
	GString *cadena;
	struct printer_conf *pointer=NULL;

	cadena=g_string_new(getenv("HOME"));
	g_string_append(cadena,"/.gtkpsproc");
	fichero=fopen(cadena->str,"r");

	configuracion=0;
	margen=8;
		
	if(!check_visor("evince"))
		visor_ps=g_string_new("evince");
	else if(!check_visor("ggv"))
		visor_ps=g_string_new("ggv");
	else if(!check_visor("gv"))
		visor_ps=g_string_new("gv");
	else {			
		noviewer_window=glade_xml_get_widget(xml,"noviewer");
		gtk_widget_show(noviewer_window);
	}
	
	if(fichero==NULL) {
		return;
	}

	configuracion=0;
	margen=8;
		
	if(!check_visor("evince"))
		visor_ps=g_string_new("evince");
	else if(!check_visor("ggv"))
		visor_ps=g_string_new("ggv");
	else if(!check_visor("gv"))
		visor_ps=g_string_new("gv");
	else {			
		noviewer_window=glade_xml_get_widget(xml,"noviewer");
		gtk_widget_show(noviewer_window);
	}
	
	pointer=NULL;
	while(1) {
		size=getline(&buffer,&bsize,fichero);
		if (size<0) {
			break;
		}
		
		if (size==0) {
			continue;
		}
		
		if (buffer[size-1]<32) {
			buffer[size-1]=0;
			size--;
		}
		
		if ((size>7) && (!strncmp(buffer,"viewer:",7))) {
			g_string_free(visor_ps,TRUE);
			visor_ps=g_string_new(buffer+7);
		}
		
		if ((size>8) && (!strncmp(buffer,"printer:",8))) {
			pointer=set_printer((guchar *)buffer+8,configuration_list);
			if (pointer) {
				pointer->configurada=1;
			}
		}
		
		if ((size>7) && (!strncmp(buffer,"margin:",7))) {
			if (pointer) {
				pointer->margen=atoi(buffer+7);
			}
		}
		
		if ((size>7) && (!strncmp(buffer,"config:",7))) {
			if (pointer) {
				pointer->configuracion=atoi(buffer+7);
			}
		}
		
	}

	fclose(fichero);
}

void set_preview_buttons() {
	
	GtkWidget *button1,*button2;
	gboolean estado;
	const gchar *impresora;
	
	button1=glade_xml_get_widget(xml,"preview_odd");
	button2=glade_xml_get_widget(xml,"preview_even");	
	
	if(GTK_TOGGLE_BUTTON (glade_xml_get_widget(xml,"parimpar"))->active) {
		gtk_widget_show(button2);
		gtk_button_set_label(GTK_BUTTON(button1),_("Preview odd pages"));
	} else {
		gtk_button_set_label(GTK_BUTTON(button1),_("Preview document"));
		gtk_widget_hide(button2);
	}
	
	if(viewer_found)
		estado=TRUE; // if it's found, allow to use the buttons
	else
		estado=FALSE;
	
	gtk_widget_set_sensitive(button1,estado);
	gtk_widget_set_sensitive(button2,estado);
}
		

int checkexist(gchar *fichero) {
	
	struct stat estado;	
	int retorno;
	
	retorno=stat(fichero,&estado);
	if(retorno==0)
		return 0;
	else
		return 1;	
}

struct printer_conf *new_printer(gchar *name) {
	
	struct printer_conf *pointer;
	
	pointer=(struct printer_conf *)g_malloc(sizeof(struct printer_conf));
	pointer->name=g_string_new(name);
	pointer->margen=8;
	pointer->configuracion=0;
	pointer->configurada=0;
	pointer->isdefault=0;
	return (pointer);
}

void fill_printers() {
	
	cups_dest_t *printers;
	int num,bucle;
	GString *cadena;
	struct printer_conf *pointer;
	
	num=cupsGetDests(&printers);

	cadena=g_string_new("");
#ifdef DEBUG
	g_print("Filling the printer list with %d printers\n",num);
#endif
	if ((num>1)||((num==1)&&(0!=strcmp("GtkPSproc",printers[0].name)))) {
		gtk_combo_box_remove_text(GTK_COMBO_BOX(wconfprinter),0);
		for(bucle=0;bucle<num;bucle++) {
			if (0!=strcmp(printers[bucle].name,"GtkPSproc")) {
#ifdef DEBUG
				g_print("Adding printer %s\n",printers[bucle].name);
#endif
				pointer=new_printer(printers[bucle].name);
				pointer->next=configuration_list;
				configuration_list=pointer;
				if (printers[bucle].is_default) {
					pointer->isdefault=1;
				}
			}
		}
	}

#ifdef DEBUG
	g_assert(configuration_list!=NULL);
#endif
	
	load_config();

#ifdef DEBUG
	g_assert(configuration_list!=NULL);
#endif

	for(pointer=configuration_list;pointer!=NULL;pointer=pointer->next) {
		if (pointer->configurada) {
			g_string_printf(cadena," %s",pointer->name->str);
		} else {
			g_string_printf(cadena,"*%s %s",pointer->name->str,_("(not configured)"));
		}
		if (pointer->isdefault) {
			gtk_combo_box_prepend_text(GTK_COMBO_BOX(wconfprinter),pointer->name->str);
		} else {
			gtk_combo_box_append_text(GTK_COMBO_BOX(wconfprinter),pointer->name->str);
		}
	}
#ifdef DEBUG
	g_assert(configuration_list!=NULL);
#endif
}

int main (int argc, char *argv[]) {

	int retorno;
	GtkWidget *ventana_error,*nombre_job;
	GString *copiador;
	
	FILE *errfichero;
	
	errfichero=fopen("/var/tmp/main.txt","w");
	fclose(errfichero);
	
	to_delete=0; /* no files to delete */
	noborrar=0; /* by default, delete temporary files */

	num_copias=1;

	configuration_list=NULL;

	bindtextdomain ("gtkpsproc", "/usr/share/locale");
	bind_textdomain_codeset ("gtkpsproc", "UTF-8");
	textdomain ("gtkpsproc");
	
	gtk_set_locale ();
	
	procesa_parametros(argc,argv);
	
	if (FALSE==gtk_init_check (&argc, &argv)) {
		printf(_("Can't init the GTK system\n"));
		exit(0);
	}
	glade_init();
	xml = glade_xml_new ("/usr/share/gtkpsproc/gtkpsproc.glade", NULL,NULL);
	if (xml==NULL) {
		printf(_("Error: gtkpsproc.glade file not found.\n"));
		exit(1);
	}
	//xml = glade_xml_new ("./gtkpsproc.glade", NULL,NULL);
	
	/* This is important */
	glade_xml_signal_autoconnect (xml);

/* here we check that the PSUTILS are installed in the system */	
	
	retorno=0;
	retorno+=checkexist("/usr/bin/pstops");
	retorno+=checkexist("/usr/bin/psnup");
	retorno+=checkexist("/usr/bin/psbook");	
	
	if(retorno!=0) {
		ventana_error = glade_xml_get_widget(xml,"nopsutils");
		gtk_widget_show (ventana_error);
	} else {
		evenpagesOK=about_window=first_time_window=config_window=NULL;
		mainwindow = glade_xml_get_widget(xml,"mainwindow");
		config_window=glade_xml_get_widget(xml,"configuracion");
		wconfprinter=glade_xml_get_widget(xml,"confprinter");
		fill_printers();
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"copias")),num_copias);
		
		fichero=g_string_new("");
  		g_string_sprintf(fichero,"/var/tmp/gtkpsproc%d%d",(int)time(NULL),getpid());
		fichero_temporal=g_string_new("");
  		g_string_sprintf(fichero_temporal,"%s_temporal.ps",fichero->str);

		copiador=g_string_new(""); /* copy the file to fichero_temporal */
		if(0==strcmp("/dev/stdin",ficherops->str)) {
			g_string_sprintf(copiador,"cat /dev/stdin > %s",fichero_temporal->str);	  
			run_command(copiador->str);
			g_string_sprintf(ficherops,"%s",fichero_temporal->str);
			to_delete+=FTEMPORAL; /* now, fichero_temporal must be deleted */
			g_string_free(copiador,TRUE);
		}
		nombre_job=glade_xml_get_widget(xml,"job_title");
		gtk_label_set_text(GTK_LABEL(nombre_job),job_ps->str);
		check_for_visor();
		gtk_widget_show (mainwindow);
		set_preview_buttons();
		get_ps_size(ficherops,&thepapersize);
		printf("Found: %d\nSize: %d %d   %d %d   %d %d  %d %d\n",thepapersize.found,thepapersize.pwidth,thepapersize.pheigth,thepapersize.dwidth,thepapersize.dheigth,thepapersize.swidth,thepapersize.sheigth,thepapersize.bwidth,thepapersize.bheigth);
		printf("Final size: %d %d\n",thepapersize.width,thepapersize.heigth);
	}
	gtk_main ();
	return 0;
}
